@@ -2,10 +2,10 @@
 /*
  * gsf-input-stdio.c: stdio based input
  *
- * Copyright (C) 2002 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2003 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2.1 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -50,7 +50,7 @@ typedef struct {
  *
  * Returns a new file or NULL.
  **/
-GsfInput *
+GsfInputStdio *
 gsf_input_stdio_new (char const *filename, GError **err)
 {
 	GsfInputStdio *input;
@@ -89,7 +89,7 @@ gsf_input_stdio_new (char const *filename, GError **err)
 	gsf_input_set_size (GSF_INPUT (input), size);
 	gsf_input_set_name (GSF_INPUT (input), filename);
 
-	return GSF_INPUT (input);
+	return input;
 }
 
 static void
@@ -117,7 +117,7 @@ static GsfInput *
 gsf_input_stdio_dup (GsfInput *src_input, GError **err)
 {
 	GsfInputStdio const *src = (GsfInputStdio *)src_input;
-	return gsf_input_stdio_new (src->input.name, err);
+	return GSF_INPUT (gsf_input_stdio_new (src->input.name, err));
 }
 
 static guint8 const *
@@ -125,7 +125,7 @@ gsf_input_stdio_read (GsfInput *input, size_t num_bytes,
 		      guint8 *buffer)
 {
 	GsfInputStdio *stdio = GSF_INPUT_STDIO (input);
-	size_t res;
+	size_t nread = 0, total_read = 0;
 
 	g_return_val_if_fail (stdio != NULL, NULL);
 	g_return_val_if_fail (stdio->file != NULL, NULL);
@@ -135,14 +135,19 @@ gsf_input_stdio_read (GsfInput *input, size_t num_bytes,
 			stdio->buf_size = num_bytes;
 			if (stdio->buf != NULL)
 				g_free (stdio->buf);
-			stdio->buf = g_malloc (stdio->buf_size);
+			stdio->buf = g_new (guint8, stdio->buf_size);
 		}
 		buffer = stdio->buf;
 	}
 
-	res = fread (buffer, 1, num_bytes, stdio->file);
-	if (res < num_bytes)
-		return NULL;
+	while (total_read < num_bytes) {
+		nread = fread (buffer + total_read, 1, 
+			       num_bytes - total_read, stdio->file);
+		total_read += nread;
+		if (total_read < num_bytes &&
+		    (ferror (stdio->file) || feof (stdio->file)))
+			return NULL;
+	}
 
 	return buffer;
 }
@@ -152,6 +157,7 @@ gsf_input_stdio_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 {
 	GsfInputStdio const *stdio = GSF_INPUT_STDIO (input);
 	long loffset;
+	int stdio_whence = SEEK_SET;
 
 	if (stdio->file == NULL)
 		return TRUE;
@@ -162,19 +168,16 @@ gsf_input_stdio_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 		return TRUE;
 	}
 	switch (whence) {
-	case G_SEEK_SET :
-		if (0 == fseek (stdio->file, loffset, SEEK_SET))
-			return FALSE;
+	case G_SEEK_CUR : stdio_whence = SEEK_CUR; break;
+	case G_SEEK_END : stdio_whence = SEEK_END; break;
+	case G_SEEK_SET:
+	default:
 		break;
-	case G_SEEK_CUR :
-		if (0 == fseek (stdio->file, loffset, SEEK_CUR))
-			return FALSE;
-		break;
-	case G_SEEK_END :
-		if (0 == fseek (stdio->file, loffset, SEEK_END))
-			return FALSE;
 	}
 
+	if (0 == fseek (stdio->file, loffset, stdio_whence))
+		return FALSE;
+	
 	return TRUE;
 }
 
